# Lab-1

1. 原理：在 `rust_main` 函数中，执行 `ebreak` 命令后至函数结束前，`sp` 寄存器的值是怎样变化的？

    ebreak命令会触发Breakpoint Trap, 因此首先进入中断处理流程

    由代码所示, 我们注册的中断处理入口是__interrupt(),  该入口在interrupt.asm中, 所以我们要看__interrupt函数是如何操作寄存器的.

    ```rust
    pub fn init() {
        unsafe {
            extern "C" {
                /// `interrupt.asm` 中的中断入口
                fn __interrupt();
            }
            // 使用 Direct 模式，将中断入口设置为 `__interrupt`
            stvec::write(__interrupt as usize, stvec::TrapMode::Direct);

            // 开启外部中断使能
            sie::set_sext();
        }
    }
    ```

    可以看到汇编函数首先开辟了一块栈空间Context用来保存寄存器, 包括31个通用寄存器, sp, sstatus, sepc 一共34个寄存器, 一共需要34*8=272B. 由于栈底在高地址, 栈顶在低地址, 所以我们需要将 sp减去272, 这样就开辟了272B的栈空间. 保存寄存器后, 该函数又调用了handle_interrupt函数, 因此继续看handle_interrupt函数.

    由于局部变量是放在栈中的, 因此sp可能会被移动以存入局部变量, 根据代码可以看出, handle_interrupt函数以及ebreak命令的handler也就是breakpoint函数, 是没有直接操作sp的, 因此函数退出, sp增加, 局部变量被清除掉. 

    handle_interrupt函数返回, 程序就又回到了interrupt.asm文件, pc继续向下移动, 开始执行__restore函数, 该函数把__interrupt存的所有寄存器恢复到寄存器中, 然后增加sp的值, 删除之前开辟的272B的空间, 至此llvm_asm!("ebreak")的调用结束

2. 分析：如果去掉 `rust_main` 后的 `panic` 会发生什么，为什么？

    这个问题在做实验前就遇到了, 当时在main函数 println 之后, 没有调用shutdown关机, 因此就一直在打印重复的话出来, 刚开始我以为是不是没有关机然后qemu出了什么问题就不停地读输出缓冲区, 导致满屏幕都是打印的内容, 后来想想肯定不是, 因为程序压根没有结束, 反汇编之后发现并没有循环代码, 那肯定不是我写的代码导致循环的. 之后翻文档时看到裸机两个字, 恍然大悟, 裸机是没有内存保护的, pc只会不停地往后移动, 因此应该是链接在我的程序后面的代码导致了循环.

3. 实验
    1. 如果程序访问不存在的地址，会得到 `Exception::LoadFault`。模仿捕获 `ebreak` 和时钟中断的方法，捕获 `LoadFault`（之后 `panic` 即可）。
    2. 在处理异常的过程中，如果程序想要非法访问的地址是 `0x0`，则打印 `SUCCESS!`。
    3. 添加或修改少量代码，使得运行时触发这个异常，并且打印出 `SUCCESS!`。

    见lab1代码