use crate::Allocator;
use alloc::vec::{Vec};

pub struct Segment {
    left: usize,
    right: usize,
    mid: usize,
    remains: usize
}
impl Segment {
    pub fn new(left: usize, right: usize) -> Self {
        Segment{
            left,
            right,
            mid: (right+left)/2,
            remains: right-left+1
        }
    }
    pub fn leaf(&self) -> bool {
        self.left==self.right
    }
}

pub struct SegmentTreeAllocator {
    tree: Vec<Option<Segment>>,
    len: usize
}

impl Allocator for SegmentTreeAllocator {
    fn new(capacity: usize) -> Self {
        let mut tree = Vec::new();
        let len = capacity * 2 - 1;
        for _i in 0..len {
            tree.push(None);
        }
        let mut tree = SegmentTreeAllocator { tree, len };
        tree.build(0, 0, capacity);
        tree
    }

    fn alloc(&mut self) -> Option<usize> {
        self.find(0)
    }

    fn dealloc(&mut self, index: usize) {
        self.give_back(index,0);
    }
}
impl SegmentTreeAllocator {
    pub fn build(&mut self, index: usize, left: usize, right: usize) {
        if index >= self.len {
            return;
        }
        //叶子节点
        let seg = Segment::new(left,right);
        if left == right {
            self.tree[index] = Some(seg);
            return;
        }
        let mid = seg.mid;
        self.tree[index] = Some(seg);
        self.build(self.left_child(index), left, mid);
        self.build(self.right_child(index), mid+1, right);
    }
    pub fn left_child(&self, index: usize) -> usize {
        index*2+1
    }
    pub fn right_child(&self, index: usize) -> usize {
        index*2+2
    }

    pub fn find(&mut self, index: usize) -> Option<usize> {
        if index >= self.len {
            return None;
        }
        let root = self.tree[index].as_mut().unwrap();
        //不足，直接退出
        if root.remains == 0 {
            return None;
        }
        root.remains -= 1;
        if root.leaf() {
            //返回其序号，而不是数组中的index
            return Some(root.left);
        }
        let left_child = self.left_child(index);
        let left_res = self.find(left_child);
        if left_res.is_some() {
            return left_res;
        }
        let right_child = self.right_child(index);
        let right_res = self.find(right_child);
        if right_res.is_some() {
            return right_res;
        }
        return None;
    }

    //归还
    pub fn give_back(&mut self, target: usize, cur_idx: usize) -> bool {
        if cur_idx >= self.len {
            return false;
        }
        let cur = self.tree[cur_idx].as_mut().unwrap();
        if (cur.right - cur.left) == cur.remains {
            return false;ya
        }
        cur.remains += 1;
        //找到了节点
        if cur.leaf() && cur.mid == target {
            return true;
        }
        //在左侧
        if target <= cur.mid {
            if self.give_back(target, self.left_child(cur_idx)) {
                return true;
            }
        }else {
            if self.give_back(target, self.right_child(cur_idx)) {
                return true;
            }
        }
        false
    }

}