//! 进程相关的内核功能

use super::*;
use crate::fs::{ROOT_INODE, INode, FsError, FileType};
use alloc::string::String;
use alloc::boxed::Box;
use core::ptr::slice_from_raw_parts;
use core::slice::from_raw_parts_mut;

pub(super) fn sys_exit(code: usize) -> SyscallResult {
    println!(
        "thread {} exit with code {}",
        PROCESSOR.lock().current_thread().id,
        code
    );
    SyscallResult::Kill
}

pub(super) fn sys_get_tid() -> SyscallResult {
    let thread: Arc<Thread> = PROCESSOR.lock().current_thread();
    SyscallResult::Proceed(thread.id)
}

pub(super) fn sys_fork(context: &mut Context) -> SyscallResult {
    let thread = PROCESSOR.lock().current_thread();
    let sub_process = thread.process.fork();
    context.x[10] = 0;
    let process_id = sub_process.id.clone();
    let sub_thread = thread.fork(*context, sub_process).unwrap();
    PROCESSOR.lock().add_thread(sub_thread);
    SyscallResult::Proceed(process_id)
}

pub(super) fn sys_open(name: &str) -> SyscallResult {
    // 从文件系统中找到程序
    match ROOT_INODE.find(name) {
        Ok(file) => {
            let thread = PROCESSOR.lock().current_thread();
            let mut desc = thread.process.inner.lock();
            let len = desc.descriptors.len();
            desc.descriptors.push(file);
            SyscallResult::Proceed((len-1) as isize)
        }
        Err(err) => {
            println!("{}", err);
            SyscallResult::Proceed(-1)
        }
    }
}

pub(super) fn sys_pipe(fd: *mut usize, size: usize) -> SyscallResult {
    let fd = unsafe { from_raw_parts_mut(fd, size) };
    let thread = PROCESSOR.lock().current_thread();
    let process = thread.process.clone();
    //创建两个文件描述符
    let read = ROOT_INODE.create("read", FileType::File, 0).unwrap();
    let write = ROOT_INODE.create("write", FileType::File, 0).unwrap();
    //添加文件描述符
    let desc = &mut process.inner.lock().descriptors;
    desc.push(write);
    fd[0] = desc.len()-1;
    desc.push(read);
    fd[1] = desc.len()-1;
    SyscallResult::Proceed(1)
}

pub(super) fn sys_close(fd: usize) -> SyscallResult {
    SyscallResult::Kill
}